@@ -178,9 +178,12 @@ class SPARK_CONFIG:
 
 class MODEL_SERVING:
     MODELS_DATASET = "Models"
-    SERVING_TYPE_TENSORFLOW = "TENSORFLOW"
-    SERVING_TYPE_SKLEARN = "SKLEARN"
-    SERVING_TYPES = [SERVING_TYPE_TENSORFLOW, SERVING_TYPE_SKLEARN]
+    MODEL_SERVER_TENSORFLOW_SERVING = "TENSORFLOW_SERVING"
+    MODEL_SERVER_FLASK = "FLASK"
+    MODEL_SERVERS = [MODEL_SERVER_TENSORFLOW_SERVING, MODEL_SERVER_FLASK]
+    SERVING_TOOL_DEFAULT = "DEFAULT"
+    SERVING_TOOL_KFSERVING = "KFSERVING"
+    SERVING_TOOLS = [SERVING_TOOL_DEFAULT, SERVING_TOOL_KFSERVING]
     SERVING_ACTION_START = "START"
     SERVING_ACTION_STOP = "STOP"
     SERVING_ACTIONS = [SERVING_ACTION_STOP, SERVING_ACTION_STOP]
@@ -506,7 +509,8 @@ class REST_CONFIG:
     JSON_SERVING_ARTIFACT_PATH = "artifactPath"
     JSON_SERVING_NAME = "name"
     JSON_SERVING_CREATOR = "creator"
-    JSON_SERVING_TYPE = "servingType"
+    JSON_MODEL_SERVER = "modelServer"
+    JSON_SERVING_TOOL = "servingTool"
     JSON_SERVING_MODEL_VERSION = "modelVersion"
     JSON_SERVING_CREATED = "created"
     JSON_SERVING_REQUESTED_INSTANCES = "requestedInstances"
