@@ -4,8 +4,6 @@ in the project.
 """
 
 from hops import hdfs, constants, util, exceptions, kafka
-from hops.experiment_impl.util import experiment_utils
-import os
 import json
 import re
 
@@ -160,51 +158,58 @@ def _start_or_stop_serving_rest(serving_id, action):
                                                             response.reason, error_code, error_msg, user_msg))
 
 
-def create_or_update(artifact_path, serving_name, serving_type="TENSORFLOW", model_version=1,
+def create_or_update(serving_name, artifact_path, model_version=1, model_server=None, kfserving=False,
                              batching_enabled = False, topic_name="CREATE",  num_partitions = 1, num_replicas = 1,
                              instances = 1):
     """
     Creates a serving in Hopsworks if it does not exist, otherwise update the existing one.
+    In case model server is not specified, it is inferred from the artifact files.
 
     Example use-case:
 
     >>> from hops import serving
-    >>> serving.create_or_update("/Models/mnist", "mnist", "TENSORFLOW", 1)
+    >>> serving.create_or_update("mnist", "/Models/mnist", 1)
 
     Args:
-        :artifact_path: path to the artifact to serve (tf model dir or sklearn script)
         :serving_name: name of the serving to create
-        :serving_type: type of the serving, e.g "TENSORFLOW" or "SKLEARN"
+        :artifact_path: path to the artifact to serve (tf model dir or python script implementing the Predict class)
         :model_version: version of the model to serve
+        :model_server: name of the model server to deploy, e.g "TENSORFLOW_SERVING" or "FLASK"
+        :kfserving: boolean flag whether to serve the model using KFServing serving tool
         :batching_enabled: boolean flag whether to enable batching for the inference requests
         :instances: the number of serving instances (the more instances the more inference requests can
         be served in parallel)
+        :kfserving: boolean flag whether to serve the model using KFServing
 
     Returns:
           None
     """
     serving_id = get_id(serving_name)
     artifact_path = hdfs._expand_path(artifact_path)
-    _validate_user_serving_input(artifact_path, serving_name, serving_type, model_version, batching_enabled,
-                                 num_partitions, num_replicas, instances)
+    if model_server is None:
+        model_server = _detect_model_server(artifact_path)
+
+    _validate_user_serving_input(serving_name, artifact_path, model_version, model_server, kfserving, batching_enabled,
+                                 topic_name, num_partitions, num_replicas, instances)
     artifact_path = hdfs.get_plain_path(artifact_path)
-    print("Creating a serving for model {} ...".format(serving_name))
-    _create_or_update_serving_rest(artifact_path, serving_name, serving_type, model_version, batching_enabled,
+    print("Creating serving {} for artifact {} ...".format(serving_name, artifact_path))
+    _create_or_update_serving_rest(serving_name, artifact_path, model_version, model_server, kfserving, batching_enabled,
                                    topic_name, num_partitions, num_replicas, serving_id, instances)
-    print("Serving for model {} successfully created".format(serving_name))
+    print("Serving {} successfully created".format(serving_name))
 
 
-def _validate_user_serving_input(model_path, model_name, serving_type, model_version, batching_enabled,
+def _validate_user_serving_input(serving_name, model_path, model_version, model_server, kfserving, batching_enabled, topic_name,
                                  num_partitions, num_replicas, instances):
     """
     Validate user input on the client side before sending REST call to Hopsworks (additional validation will be done
     in the backend)
 
     Args:
+        :serving_name: the name of the serving to create
         :model_path: path to the model or artifact being served
-        :model_name: the name of the serving to create
-        :serving_type: the type of serving
         :model_version: version of the serving
+        :model_server: name of the model server to deploy, e.g "TENSORFLOW_SERVING" or "FLASK"
+        :kfserving: boolean flag whether to serve the model using KFServing serving tool
         :batching_enabled: boolean flag whether to enable batching for inference requests to the serving
         :num_partitions: kafka partitions
         :num_replicas: kafka replicas
@@ -218,18 +223,18 @@ def _validate_user_serving_input(model_path, model_name, serving_type, model_ver
         :ValueError: if the serving input failed the validation
     """
     name_pattern = re.compile("^[a-zA-Z0-9]+$")
-    if len(model_name) > 256 or model_name == "" or not name_pattern.match(model_name):
+    if len(serving_name) > 256 or serving_name == "" or not name_pattern.match(serving_name):
         raise ValueError("Name of serving cannot be empty, cannot exceed 256 characters and must match the regular "
-                         "expression: ^[a-zA-Z0-9]+$, the provided name: {} is not valid".format(model_name))
+                         "expression: ^[a-zA-Z0-9]+$, the provided name: {} is not valid".format(serving_name))
     if not hdfs.exists(model_path):
         raise ValueError("The model/artifact path must exist in HDFS, the provided path: {} "
                          "does not exist".format(model_path))
-    if serving_type not in constants.MODEL_SERVING.SERVING_TYPES:
-        raise ValueError("The provided serving_type: {} is not supported, supported "
-                         "serving types are: {}".format(serving_type, ",".join(constants.MODEL_SERVING.SERVING_TYPES)))
+    if model_server not in constants.MODEL_SERVING.MODEL_SERVERS:
+        raise ValueError("The provided model_server: {} is not supported, supported "
+                         "model servers are: {}".format(model_server, ",".join(constants.MODEL_SERVING.MODEL_SERVERS)))
     if not isinstance(model_version, int):
         raise ValueError("The model version must be an integer, the provided version is not: {}".format(model_version))
-    if serving_type == constants.MODEL_SERVING.SERVING_TYPE_TENSORFLOW:
+    if model_server == constants.MODEL_SERVING.MODEL_SERVER_TENSORFLOW_SERVING:
         if not isinstance(num_replicas, int):
             raise ValueError("Number of kafka topic replicas must be an integer, the provided num replicas "
                              "is not: {}".format(model_version))
@@ -239,22 +244,28 @@ def _validate_user_serving_input(model_path, model_name, serving_type, model_ver
         if not isinstance(batching_enabled, bool):
             raise ValueError("Batching enabled must be a boolean, the provided value "
                              "is not: {}".format(batching_enabled))
+        if kfserving and batching_enabled:
+            raise ValueError("Batching requests is currently not supported in KFServing deployments")
+
+    if kfserving and model_server == constants.MODEL_SERVING.MODEL_SERVER_FLASK:
+        raise ValueError("Flask is currently not supported for KFServing deployments")
     if not isinstance(instances, int):
         raise ValueError("The number of serving instances must be an integer, "
                          "the provided version is not: {}".format(instances))
 
 
-def _create_or_update_serving_rest(model_path, model_name, serving_type, model_version,
-                                   batching_enabled = None, topic_name=None,  num_partitions = None,
-                                   num_replicas = None, serving_id = None, instances=1):
+def _create_or_update_serving_rest(serving_name, model_path, model_version, model_server, kfserving,
+                                   batching_enabled=None, topic_name=None, num_partitions=None,
+                                   num_replicas=None, serving_id=None, instances=1):
     """
     Makes a REST request to Hopsworks for creating or updating a model serving instance
 
     Args:
+        :serving_name: the name of the serving to create
         :model_path: path to the model or artifact being served
-        :model_name: the name of the serving to create
-        :serving_type: the type of serving
         :model_version: version of the serving
+        :model_server: name of the model server to deploy, e.g "TENSORFLOW_SERVING" or "FLASK"
+        :kfserving: boolean flag whether to serve the model using KFServing serving tool
         :batching_enabled: boolean flag whether to enable batching for inference requests to the serving
         :topic_name: name of the kafka topic ("CREATE" to create a new one, or "NONE" to not use kafka topic)
         :num_partitions: kafka partitions
@@ -262,6 +273,7 @@ def _create_or_update_serving_rest(model_path, model_name, serving_type, model_v
         :serving_id: the id of the serving in case of UPDATE, if serving_id is None, it is a CREATE operation.
         :instances: the number of serving instances (the more instances the more inference requests can
         be served in parallel)
+        :kfserving: boolean flag whether to serve the model using KFServing
 
     Returns:
         None
@@ -269,11 +281,15 @@ def _create_or_update_serving_rest(model_path, model_name, serving_type, model_v
     Raises:
         :RestAPIError: if there was an error with the REST call to Hopsworks
     """
+
+    serving_tool = constants.MODEL_SERVING.SERVING_TOOL_KFSERVING if kfserving else constants.MODEL_SERVING.SERVING_TOOL_DEFAULT
+
     json_contents = {
         constants.REST_CONFIG.JSON_SERVING_MODEL_VERSION: model_version,
         constants.REST_CONFIG.JSON_SERVING_ARTIFACT_PATH: model_path,
-        constants.REST_CONFIG.JSON_SERVING_TYPE: serving_type,
-        constants.REST_CONFIG.JSON_SERVING_NAME: model_name,
+        constants.REST_CONFIG.JSON_MODEL_SERVER: model_server,
+        constants.REST_CONFIG.JSON_SERVING_TOOL: serving_tool,
+        constants.REST_CONFIG.JSON_SERVING_NAME: serving_name,
         constants.REST_CONFIG.JSON_SERVING_KAFKA_TOPIC_DTO: {
             constants.REST_CONFIG.JSON_KAFKA_TOPIC_NAME: topic_name,
             constants.REST_CONFIG.JSON_KAFKA_NUM_PARTITIONS: num_partitions,
@@ -283,7 +299,7 @@ def _create_or_update_serving_rest(model_path, model_name, serving_type, model_v
     }
     if serving_id is not None:
         json_contents[constants.REST_CONFIG.JSON_SERVING_ID] = serving_id
-    if serving_type == constants.MODEL_SERVING.SERVING_TYPE_TENSORFLOW:
+    if model_server == constants.MODEL_SERVING.MODEL_SERVER_TENSORFLOW_SERVING:
         json_contents[constants.REST_CONFIG.JSON_SERVING_BATCHING_ENABLED] = batching_enabled
     json_embeddable = json.dumps(json_contents)
     headers = {constants.HTTP_CONFIG.HTTP_CONTENT_TYPE: constants.HTTP_CONFIG.HTTP_APPLICATION_JSON}
@@ -303,6 +319,15 @@ def _create_or_update_serving_rest(model_path, model_name, serving_type, model_v
                                       "user msg: {}".format(resource_url, response.status_code, response.reason,
                                                             error_code, error_msg, user_msg))
 
+
+def _detect_model_server(artifact_path):
+    model_server = constants.MODEL_SERVING.MODEL_SERVER_TENSORFLOW_SERVING
+    if artifact_path.endswith(".py"):
+        model_server = constants.MODEL_SERVING.MODEL_SERVER_FLASK
+    print("Inferring model server from artifact files: {}".format(model_server))
+    return model_server
+
+
 def get_id(serving_name):
     """
     Gets the id of a serving with a given name
@@ -346,24 +371,44 @@ def get_artifact_path(serving_name):
     return serving.artifact_path
 
 
-def get_type(serving_name):
+def get_model_server(serving_name):
     """
-    Gets the type of a serving with a given name
+    Gets the type of model server of the serving with a given name
 
     Example use-case:
 
     >>> from hops import serving
-    >>> serving.get_type(serving_name)
+    >>> serving.get_model_server(serving_name)
 
     Args:
-        :serving_name: name of the serving to get the typ for
+        :serving_name: name of the serving to get the type for
 
     Returns:
-         the type of the serving (e.g Tensorflow or SkLearn)
+         the model server (e.g Tensorflow Serving or Flask)
     """
     servings = get_all()
     serving = _find_serving_with_name(serving_name, servings)
-    return serving.serving_type
+    return serving.model_server
+
+
+def get_serving_tool(serving_name):
+    """
+    Gets the serving tool of the serving with a given name
+
+    Example use-case:
+
+    >>> from hops import serving
+    >>> serving.get_serving_tool(serving_name)
+
+    Args:
+        :serving_name: name of the serving to get the mode for
+
+    Returns:
+         the serving tool (e.g DEFAULT or KFSERVING)
+    """
+    servings = get_all()
+    serving = _find_serving_with_name(serving_name, servings)
+    return serving.serving_tool
 
 
 def get_version(serving_name):
@@ -579,7 +624,8 @@ class Serving(object):
         self.name = serving_json[constants.REST_CONFIG.JSON_SERVING_NAME]
         self.creator = serving_json[constants.REST_CONFIG.JSON_SERVING_CREATOR]
         self.creator = serving_json[constants.REST_CONFIG.JSON_SERVING_CREATOR]
-        self.serving_type = serving_json[constants.REST_CONFIG.JSON_SERVING_TYPE]
+        self.model_server = serving_json[constants.REST_CONFIG.JSON_MODEL_SERVER]
+        self.serving_tool = serving_json[constants.REST_CONFIG.JSON_SERVING_TOOL]
         self.model_version = serving_json[constants.REST_CONFIG.JSON_SERVING_MODEL_VERSION]
         self.created = serving_json[constants.REST_CONFIG.JSON_SERVING_CREATED]
         self.requested_instances = serving_json[constants.REST_CONFIG.JSON_SERVING_REQUESTED_INSTANCES]
